/*
 * Produced by:
 *	Peter Vale
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	CPD_Clip.C (Clip Public Domain Library, C++)
 *
 * COMMENTS:
 *
 *		A collection of raw data 
 */

#include <iostream.h>
#include <string.h>

#include "CPD_Clip.h"
#include "CPD_Track.h"

////////////////////////////////////////////////////////////////////

CPD_Clip::CPD_Clip()
{
    tracks = 0;
    xquaternion = 0;
    yquaternion = 0;
    zquaternion = 0;
    reset();
}

CPD_Clip::~CPD_Clip()
{
    reset();
}

void
CPD_Clip::reset(void)
{
    delete [] tracks;
    tracks = 0;
    number_tracks = 0;
    
    delete [] xquaternion;
    xquaternion = 0;
    delete [] yquaternion;
    yquaternion = 0;
    delete [] zquaternion;
    zquaternion = 0;
    number_quaternions = 0;
    strcpy(quaternion_order, "xyz");

    track_length = 1;
    sample_rate	= 30.0f;
    start	=  0.0f;
}


////////////////////////////////////////////////////////////////////
//
// I/O methods
//
////////////////////////////////////////////////////////////////////


// Usually use cout as the ostream parameter

int
CPD_Clip::save(ostream &os) const
{
    int		i;

    os << "{" << endl;

    os << "   rate = " << sample_rate << endl;
    os << "   start = " << start << endl;
    os << "   tracklength = " << track_length << endl;

    os << "   tracks = " << number_tracks << endl;
    for (i=0; i<number_tracks; i++)
	tracks[i].save(os, track_length);
    os << endl;

    if(number_quaternions)
    {
	os << "   quaternions = " << quaternion_order 
	   << " " << number_quaternions << endl;

	for (i=0; i<number_quaternions; i++)
	    os << "   " << xquaternion[i] 
	       << " " << yquaternion[i] 
	       << " " << zquaternion[i] << endl;
    }

    os << "}" << endl;

    return 0;
}

// Usually use cin as the istream parameter

int
CPD_Clip::load(istream &is)
{	
    int		 i;
    char	 tmp_str[512];
    char	 equal_str[512];

    tmp_str[0] =   0;	// avoid unitialized memory 
    equal_str[0] = 0;

    reset();

    is >> tmp_str;
    if (strcmp(tmp_str, "{") != 0)
    {
	cerr << "\rMissing Token: {" << endl;
	return -1;
    }

    do 
    {
	is >> tmp_str;
	if (!strcmp(tmp_str, "}"))
	    break;   // Successful end

	is >> equal_str;
	if (strcmp(equal_str, "=") != 0)
	{
	    cerr << "\rMissing Token: =" << endl;
	    return -1;
	}
	else if (!strcmp(tmp_str, "rate"))        is >> sample_rate;
	else if (!strcmp(tmp_str, "start"))       is >> start;
	else if (!strcmp(tmp_str, "tracklength")) is >> track_length;
	else if (!strcmp(tmp_str, "tracks"))
	{
	    is >> number_tracks;
	    tracks = new CPD_Track[number_tracks];

	    for (i=0; i<number_tracks; i++)
		tracks[i].load(is, track_length);
	}
	else if ((!strcmp(tmp_str, "slerps")) ||
		 (!strcmp(tmp_str, "quaternions")))
	{
	    is >> quaternion_order;

	    is >> number_quaternions;
	    xquaternion = new int[number_quaternions];
	    yquaternion = new int[number_quaternions];
	    zquaternion = new int[number_quaternions];

	    for (i=0; i<number_quaternions; i++)
	    {
		is >> xquaternion[i];
		is >> yquaternion[i];
		is >> zquaternion[i];
	    }
	}
	else cerr << "\rBad Token: " << tmp_str << endl;

    } while (!is.fail());

    return 0;
}

