/*%CC -o tomdisplay %
 *
 * Produced by:
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	tomdisplay.C
 *
 * COMMENTS:
 *	This application is an example of writing images to the imdisplay
 *	application.  It is intended to be used as instructional code to write
 *	display drivers for renderers etc.
 *
 *	As written, the program will write a constant color image to mplay
 */

#include <sys/types.h>
#include <netinet/in.h>
#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

//
// The magic number is the first integer of the header passed on the pipe
//

#define MAGIC	(('h'<<24)+('M'<<16)+('P'<<8)+('0'))

//
//  DATASIZE may be one of:
//	0	= Floating point data
//	1	= Unsigned char data
//	2	= Unsigned short data
//	4	= Unsigned int data
//
#define DATASIZE	1

//
//  NCHANNELS may be one of:
//	1	= A single channel image
//	3	= RGB data
//	4	= RGBA data
//  The pixel planes are packed so that the data
//  is layed out as RGBRGBRGB... not RRRGGGBBB...
//
#define NCHANNELS	1

#define XRES	320		// Image X resolution
#define YRES	240		// Image Y resolution
#define TXRES	40		// Tile X resolution
#define TYRES	30		// Tile Y resolution
#define NX	(XRES/TXRES)	// Number of X tiles
#define NY	(YRES/TYRES)	// Number of Y tiles

// Define how many bytes are required for each channel
#define DATABYTES	(DATASIZE == 0 ? sizeof(float) : DATASIZE)

static void	*theTile;

static void
buildTile()
{
    int			 bytes;
    int			 i, c;
    unsigned short	*sdata;
    unsigned char	*cdata;
    unsigned int	*idata;
    float		*fdata;

    // Allocate storage for the tile
    theTile = ::malloc(TXRES * TYRES * NCHANNELS * DATABYTES);

    cdata = (unsigned char *)theTile;
    sdata = (unsigned short *)theTile; 
    idata = (unsigned int *)theTile; 
    fdata = (float *)theTile;

    // Just fill in with some constant values.
    for (i = TXRES*TYRES; i-- > 0; )
    {
	for (c = NCHANNELS; c-- > 0; )
	{
		 if (DATASIZE == 1) *cdata++ = 0x85;
	    else if (DATASIZE == 2) *sdata++ = 0x8512;
	    else if (DATASIZE == 4) *idata++ = 0x85123456;
	    else		    *fdata++ = .763;
	}
    }
}

static void
writeTile(FILE *fp, int tx0, int tx1, int ty0, int ty1)
{
    int		tile_head[4];
    size_t	size;

    // Fill out the tile header
    tile_head[0] = tx0;
    tile_head[1] = tx1;
    tile_head[2] = ty0;
    tile_head[3] = ty1;
    size = (tx1 - tx0 + 1) * (ty1 - ty0 + 1);

    // Write the tile header to the pipe
    if (fwrite(tile_head, sizeof(int), 4, fp) != 4)
    {
	::fprintf(stderr, "Error writing tile: %d %d %d %d\n",
			tx0, tx1, ty0, ty1);
	::exit(1);
    }
    // Write the data to the pipe
    if (fwrite(theTile, DATABYTES*NCHANNELS, size, fp) != size)
    {
	::fprintf(stderr, "Error writing tile: %d %d %d %d\n",
			tx0, tx1, ty0, ty1);
	::exit(1);
    }
}

int
main(int argc, char *argv[])
{
    char	 cmd[512];
    int		 header[8];
    int		 tx, ty;
    FILE	*fp;

    ::strcpy(cmd, "imdisplay -p");
    ::strcat(cmd, " -n tomdisplay");	// Set the name of the image
    //
    // If the first scanline of the image is at the top, the -f option should
    // be specified.
    //::strcat(cmd, " -f");			// Flip the image

    fp = ::popen(cmd, "wb");
    if (!fp)
    {
	::fprintf(stderr, "Unable to open imdisplay\n");
	::exit(1);
    }

    ::memset(header, 0, sizeof(header));
    header[0] = MAGIC;
    header[1] = XRES;
    header[2] = YRES;
    header[3] = DATASIZE;
    header[4] = NCHANNELS;

    if (fwrite(header, sizeof(int), 8, fp) != 8)
    {
	::fprintf(stderr, "Unable to write header\n");
	::exit(1);
    }

    ::buildTile();		// Build the tile to write
    for (ty = 0; ty < YRES; ty += TYRES)
	for (tx = 0; tx < XRES; tx += TXRES)
	    ::writeTile(fp, tx, tx+TXRES-1, ty, ty+TYRES-1);

    ::pclose(fp);
    ::free(theTile);
}
