# defaultscene.cmd : Houdini script for creating the default scene on start up
#
# This file is sourced from 123.cmd when Houdini starts up to create the
# default scene.
#

# save our current path
set saved_path = `execute("oppwf")`

#####################################
# Create the objects
#

opcf /obj

#
#  When we add the geometry, don't run the creation scripts (-n option)
#
opcf /obj

opadd -n geo model
opparm model stdswitcher ( 0 0 0 0 0 ) use_dcolor ( 0 )
oplocate -x 1.7973 -y 2.4 model
opset -C off -p on model

#
# When adding the light sources, we will run the creation scripts
#
opadd ambient ambient1
opset -d off -r off -C off -p off ambient1
opadd  light light1
opparm light1 t ( 1.5 1.5 1.5 ) r ( -45 45 0 ) c ( 0.5 0.5 0.5 ) shader ( fastShadow )
opset -d off -r off -C off -p off light1
opadd light light2
opparm light2 t ( -1.5 1.5 1.5 ) r ( -45 -45 0 ) c ( 0.5 0.5 0.5 )
opset -d off -r off -C off -p off light2
opadd light light3
opparm light3 t ( 0 2 0 ) r (-90 0 0 ) c ( 0.5 0.5 0.5 ) lookup ( quat )
opset -d off -r off -C off -p off light3

#
# CAM1 object
#
opadd cam cam1
opparm cam1 t ( 0 0 5 ) use_dcolor( 0 )
opset -d off -r off -C off -p off cam1

#
# Position all our object tiles
#
oplocate -x 2.85     -y 0.2 ambient1
oplocate -x 2.85     -y 1.4 cam1
oplocate -x 0.306187 -y 1.4 light1
oplocate -x 0.99261  -y 0.8 light2
oplocate -x 1.68743  -y 0.2 light3

#
#  Set up the Output Drivers for Mantra
#
opcf /out
opadd mantra fast_mantra
oplocate -x 1.40707 -y 1.13636 fast_mantra
opparm fast_mantra camera ( cam1 ) tscript ( off )
opparm fast_mantra script ( "" ) binary ( on ) picture ( ip )
opparm fast_mantra dof ( off ) jitter ( 1 ) dither ( 0.004 ) sample ( 2 2 )
opparm fast_mantra field ( frame ) tres ( off ) res ( 320 243 ) aspect ( 1 )
opparm fast_mantra command ( 'mantra' ) ray_filter ( 'gaussian 1.5 1.5' )

# Add a renderer which renders only the selected objects
    opadd mantra mantra_sel
    oplocate -x 1.40707 -y 2.1 mantra_sel
    opparm mantra_sel camera ( cam1 ) tscript ( off )
    opparm mantra_sel script ( "" ) binary ( on ) picture ( ip )
    opparm mantra_sel visible ( '`opselect("/obj")`'
    opparm mantra_sel dof ( off ) jitter ( 1 ) dither ( 0.004 ) sample ( 2 2 )
    opparm mantra_sel field ( frame ) tres ( off ) res ( 320 243 ) aspect ( 1 )
    opparm mantra_sel command ( 'mantra' ) ray_filter ( 'gaussian 1.5 1.5' )

opadd mantra mantra1
oplocate -x 2.5 -y 1.13636 mantra1
opparm mantra1 camera ( cam1 ) tscript ( off )
opparm mantra1 script ( "" ) binary ( on ) picture ( ip )
opparm mantra1 dof ( off ) jitter ( 1 ) dither ( 0.004) sample ( 3 3 )
opparm mantra1 field ( frame ) tres ( off ) res ( 320 243 ) aspect ( 1 )
opparm mantra1 command ( 'mantra' ) ray_filter ( 'gaussian 1.5 1.5' )
opset -C on -p on mantra1

#
# Add some default networks for the remaining editors
#

opcf /part
opadd popnet popnet1
oplocate -x 0 -y 0 popnet1

opcf /ch
opadd ch ch1
oplocate -x 0 -y 0 ch1

opcf /img
opadd img img1
oplocate -x 0 -y 0 img1

#
#  Restore the original path location
#
opcf $saved_path

#
# Set the current object to be model geometry
#
opset -C on -p on /obj/model

#
# Now we set all the perspective views that have been loaded so far
# to use camera1 as their camera
#
set views = `run('viewls -n -t "world"')`
if ("$views" != "") then
    foreach view ( $views )
	set quads = `run('viewls -n -v -T "perspective"')`
	if ("$quads" != "") then
	    foreach quad ( $quads )
		viewcamera -c cam1 $quad
	    end
	endif
    end
endif

