#
#  This script may be used to rename all currently selected object 
#  nodes using prefix and suffix options.
#

\set noalias = 1
set saved_path = `execute("oppwf")`

if ( $argc != 6 && $argc != 7 )
   echo Usage: source $arg0 [-d] net prefix remove_prefix_str suffix remove_suffix_str
   echo 
   echo \tnet is the full network path to perform the renaming under
   echo 
   echo \tprefix is the string to use as the new prefix
   echo
   echo \tremove_prefix_str specifies which character to stop when removing
   echo \tprefixes. If this parameter is "", then prefixes are not removed.
   echo
   echo \tsuffix is the string to use as the new suffix
   echo
   echo \tremove_suffix_str specifies which character to start when removing
   echo \tprefixes. If this parameter is "", then suffixes are not removed.
   echo
   echo \tIf the -d parameter is given, then $arg0 will first try to 
   echo \tremove trailing digits.
   exit
endif

set nodigits = 0
if( "$arg1" == "-d" ) then
    set nodigits = 1
    shift
endif

opcf $arg1

foreach obj ( `opselect( "$arg1" )` )
    set oldobj = $obj

    # remove prefix
    if( `strlen( "$arg3" )` > 0 ) then
	set pos = `index( "$obj", "$arg3" )`
	set pos = `$pos + 1`
	set len = `strlen( "$obj" ) - $pos`
	set obj = `substr( "$obj", $pos, $len )`
    endif

    # remove trailing digits
    if( $nodigits == 1 ) then
	foreach digit ( "0" "1" "2" "3" "4" "5" "6" "7" "8" "9" )
	    set digitpos = `rindex( "$obj", "$digit" )`
	    if( $digitpos >= 0 ) then
		set obj = `substr( "$obj", 0, $digitpos )`
	    endif
	end
    endif

    # remove suffix 
    if( `strlen( "$arg5" )` > 0 ) then
	set pos = `rindex( "$obj", "$arg5" )`
	if( $pos >= 0 ) then
	    set obj = `substr( "$obj", 0, $pos )`
	endif
    endif

    # add prefix and suffix
    set obj = `strcat( "$arg2", strcat( "$obj", "$arg4" ) )`

    # don't rename if resulting name is zero length or doesn't change
    if( `strlen( "$obj" )` > 0 && `strcmp( "$obj", "$oldobj" )` != 0 ) then

	# make sure current name does not exist, if it does, get a unique name
	set ls = `run("opls -d $obj")`
	if ( "$ls" != "" ) then
	    for i = 1 to 10000
		set name = $obj$i
		set ls = `run("opls -d $name")`
		if ( "$ls" == "" ) then
		    set obj = $name
		    break
		endif
	    end
	endif

	opname $oldobj $obj
    endif
end

opcf $saved_path

