# Houdini-Select hselect.cmd initialization script
# This script is run if no arguments are specified to hselect

#  Create some useful aliases

# Variables set with the set command are local to this script only.
# Therefore, we don't have to worry about cleaning up.
# These variables are used to set the Function Key aliases below.
set F1 = desk set Animate
set F2 = desk set Build
set F3 = desk set Textport
set F4 = desk set Model
set F5 = desk set VEX_Builder

# Set the function key aliases.
for i = 1 to 5
    alias F${i} ${F$i}
    alias AF${i} ${F$i}
end

alias cd	opcf
alias h		history
alias ls	opls
alias pwd	oppwf
alias rm	oprm
alias cp	opcp

alias SF10 'quit'


#
# Turn off aliases for the rest of this file in case of conflicts
#
set noalias = 1

set saved_path = `execute("oppwf")`

#
#  When we add the geometry, don't run the creation scripts (-n option)
#
opcf /obj

opadd -n geo model
opparm model stdswitcher ( 0 0 0 0 0 ) use_dcolor ( 0 )
oplocate -x 1.7973 -y 5.23424 model
opset -C on -p on model

#
# When adding the light sources, we will run the creation scripts
#
opadd ambient ambient1
opset -d off -r off -C off -p off ambient1
oplocate -x 2.85207 -y 2.17693 ambient1
opadd  light light1
opparm light1 t ( 1.5 1.5 1.5 ) lookat ( model ) c ( 0.5 0.5 0.5 )
opset -d off -r off -C off -p off light1
oplocate -x 0.706187 -y 4.06249 light1
opadd light light2
opparm light2 t ( -1.5 1.5 1.5 ) lookat ( model ) c ( 0.5 0.5 0.5 )
opset -d off -r off -C off -p off light2
oplocate -x 1.39261 -y 3.44366 light2
opadd light light3
opparm light3 t ( 0 2 0 ) lookat ( model ) c ( 0.5 0.5 0.5 ) lookup ( quat )
opset -d off -r off -C off -p off light3
oplocate -x 2.08743 -y 2.8169 light3

#
# camera object
#
opadd cam cam1
opparm cam1 t ( 0 0 5 ) use_dcolor ( 0 )
opset -d off -r off -C off -p off cam1
oplocate -x 2.87009 -y 4.03553 cam1

#
#  Restore the original path location
#
opcf $saved_path

# Set the prompt to display the current directory followed by an arrow
prompt '`strcat(oppwf(), " -> ")`'

# First we try to find a good desk to be the startup desk. If you want a
# different desk to be the default, just change the logic below (it gives
# the Model desk the highest priority, followed by the Build desk).

set desks = `run("desk list")`
set model = 0
set build = 0
set animate = 0
foreach desk ( $desks )
    if ($desk == "Model")
	set model = 1
    else if ($desk == "Build")
	set build = 1
    else if ($desk == "Animate")
	set animate = 1
    endif
end

if ($model == 1) then
    desk set Model
else if ($build == 1) then
    desk set Build
else if ($animate == 1) then
    desk set Animate
endif
