# Default script run when a bone object is created by the bone state
# $arg1 is the bone name
# $arg2 is the optional link geometry 
# $arg3 is a flag for whether or not the link geometry should render

\set noalias = 1
chlock $arg1 +t? +s?

opdelscript -c 'bonefixchops $OPFULLNAME' $arg1

opcf $arg1

#
# The Bone 
#
# the following node must be named link_scale!
opadd -n xform		link_scale

opadd -n switch 	show_link
opadd -n add		point1
opadd -n switch 	show_capture
opadd -n cregion    	cregion1
opadd -n merge 		link_and_capture
opadd -n merge		cregion

if ("$arg2" != "") then
    # do not name this node to anything other than link!
    opadd -n object_merge link

    opparm link numobj( 1 ) xformpath ( "" ) enable1( on ) objpath1 ( $arg2 )
else
    opadd -n tube linkseg1
    opadd -n tube linkseg2
    opadd -n tube ikseg1
    opadd -n tube ikseg2
    opadd -n tube ikseg3
    opadd -n tube ikseg4
    opadd -n merge ikseg_merge
    opadd -n merge link_merge
    opadd -n switch link_type

    # do not name this node to anything other than link!
    opadd -n xform link
    opparm link s ( 1 2 1 )

    chadd -t $TSTART $TEND link_type input
    chkey -t $TSTART -F 'strcmp(chs("../solver"),"")!=0' link_type/input

    opparm linkseg1 type ( mesh ) orient (z) t ( 0 0 -0.05 ) rad ( 0 0.05 ) height (0.1) cols ( 4 )
    opparm linkseg2 type ( mesh ) orient (z) t ( 0 0 -0.55 ) rad ( 0.05 0 ) height (0.9) cols ( 4 )

    opparm ikseg1 type ( mesh ) orient ( z ) t ( 0 0 -0.05 ) rad ( 0 0.05 ) height ( 0.1 ) cols ( 4 )
    opparm ikseg2 type ( mesh ) orient ( z ) t ( 0 0 -0.15 ) rad ( 0.05 0.03 ) height ( 0.1 ) cols ( 4 )
    opparm ikseg3 type ( mesh ) orient ( z ) t ( 0 0 -0.25 ) rad ( 0.03 0.038 ) height ( 0.1 ) cols ( 4 )
    opparm ikseg4 type ( mesh ) orient ( z ) t ( 0 0 -0.65 ) rad ( 0.038 0 ) height ( 0.7 ) cols ( 4 )

    oplocate -x 0.0 -y 7 linkseg1
    oplocate -x 1.5 -y 7 linkseg2

    oplocate -x 3.5 -y 7 ikseg1
    oplocate -x 5.0 -y 7 ikseg2
    oplocate -x 6.5 -y 7 ikseg3
    oplocate -x 8.0 -y 7 ikseg4
    oplocate -x 5.75 -y 6 ikseg_merge

    oplocate -x 1.5 -y 6 link_merge
    oplocate -x 1.5 -y 5 link_type

    opwire linkseg1 -0 link_merge
    opwire linkseg2 -1 link_merge
    opwire link_merge -0 link_type

    opwire ikseg1 -0 ikseg_merge
    opwire ikseg2 -1 ikseg_merge
    opwire ikseg3 -2 ikseg_merge
    opwire ikseg4 -3 ikseg_merge
    opwire ikseg_merge -1 link_type
    opwire link_type -0 link
endif

chadd -t $TSTART $TEND link_scale scale
chkey -t $TSTART -F 'if( strcmp(run("objkinoverride"), "capture") == 0, ch("../captposelen"), ch("../length"))' link_scale/scale

chadd -t $TSTART $TEND show_link input
chkey -t $TSTART -F '!ch("../displaylink") || strcmp(run("objkinoverride"), "capture") == 0' show_link/input

opparm point1 usept0 ( on )

chadd -t $TSTART $TEND show_capture input
chkey -t $TSTART -F 'ch("../displaycapture") || strcmp(run("objkinoverride"), "capture") == 0' show_capture/input 


#
#  Capture/Deform capture region parms
#  
  
# capture:
chadd -t $TSTART $TEND cregion1 ccenterx ccentery ccenterz cbcapx cbcapy cbcapz ctcapx ctcapy ctcapz ctopheight cbotheight 
chadd -t $TSTART $TEND cregion1 csquashx csquashy csquashz czfactor crx cry crz
chkey -t $TSTART -F 'ch("../ccrcenterx")' cregion1/ccenterx
chkey -t $TSTART -F 'ch("../ccrcentery")' cregion1/ccentery
chkey -t $TSTART -F 'ch("../ccrcenterz")' cregion1/ccenterz
chkey -t $TSTART -F 'ch("../ccrrotatex")' cregion1/crx
chkey -t $TSTART -F 'ch("../ccrrotatey")' cregion1/cry
chkey -t $TSTART -F 'ch("../ccrrotatez")' cregion1/crz
set tmpx=ch(\"../ccrbotcapx\")
set tmpy=ch(\"../ccrbotcapy\")
set tmpz=ch(\"../ccrbotcapz\")
chkey -t $TSTART -F "$tmpx" cregion1/cbcapx
chkey -t $TSTART -F "$tmpy" cregion1/cbcapy
chkey -t $TSTART -F "$tmpz" cregion1/cbcapz
set tmpx=ch(\"../ccrtopcapx\")
set tmpy=ch(\"../ccrtopcapy\")
set tmpz=ch(\"../ccrtopcapz\")
chkey -t $TSTART -F "$tmpx" cregion1/ctcapx
chkey -t $TSTART -F "$tmpy" cregion1/ctcapy
chkey -t $TSTART -F "$tmpz" cregion1/ctcapz
chkey -t $TSTART -F 'ch("../ccrtopheight")' cregion1/ctopheight
chkey -t $TSTART -F 'ch("../ccrbotheight")' cregion1/cbotheight
chkey -t $TSTART -F 'ch("../ccrscalex")' cregion1/csquashx
chkey -t $TSTART -F 'ch("../ccrscaley")' cregion1/csquashy
chkey -t $TSTART -F 'ch("../ccrscalez")' cregion1/csquashz
chkey -t $TSTART -F 'ch("../captposelen")' cregion1/czfactor
opparm cregion1 corient ( z )

# deform:
chadd -t $TSTART $TEND cregion1 centerx centery centerz bcapx bcapy bcapz tcapx tcapy tcapz topheight botheight 
chadd -t $TSTART $TEND cregion1 squashx squashy squashz zfactor rx ry rz
chkey -t $TSTART -F 'ch("../crcenterx")' cregion1/centerx
chkey -t $TSTART -F 'ch("../crcentery")' cregion1/centery
chkey -t $TSTART -F 'ch("../crcenterz")' cregion1/centerz
chkey -t $TSTART -F 'ch("../crrotatex")' cregion1/rx
chkey -t $TSTART -F 'ch("../crrotatey")' cregion1/ry
chkey -t $TSTART -F 'ch("../crrotatez")' cregion1/rz
set tmpx=ch(\"../crbotcapx\")
set tmpy=ch(\"../crbotcapy\")
set tmpz=ch(\"../crbotcapz\")
chkey -t $TSTART -F "$tmpx" cregion1/bcapx
chkey -t $TSTART -F "$tmpy" cregion1/bcapy
chkey -t $TSTART -F "$tmpz" cregion1/bcapz
set tmpx=ch(\"../crtopcapx\")
set tmpy=ch(\"../crtopcapy\")
set tmpz=ch(\"../crtopcapz\")
chkey -t $TSTART -F "$tmpx" cregion1/tcapx
chkey -t $TSTART -F "$tmpy" cregion1/tcapy
chkey -t $TSTART -F "$tmpz" cregion1/tcapz
chkey -t $TSTART -F 'ch("../crtopheight")' cregion1/topheight
chkey -t $TSTART -F 'ch("../crbotheight")' cregion1/botheight
chkey -t $TSTART -F 'ch("../crscalex")' cregion1/squashx
chkey -t $TSTART -F 'ch("../crscaley")' cregion1/squashy
chkey -t $TSTART -F 'ch("../crscalez")' cregion1/squashz
chkey -t $TSTART -F 'ch("../length")' cregion1/zfactor
opparm cregion1 orient ( z )
  
oplocate -x 1.5 -y 4 link
oplocate -x 1.5 -y 3 link_scale
oplocate -x 2.0 -y 2 show_link
oplocate -x 3.0 -y 3 point1
oplocate -x 5.5 -y 4 cregion1
oplocate -x 6.0 -y 3 cregion
oplocate -x 4.0 -y 2 show_capture
oplocate -x 3.0 -y 1 link_and_capture

opwire link_scale 	-0 show_link
opwire point1 		-1 show_link
opwire show_link 	-0 link_and_capture
opwire show_capture 	-1 link_and_capture
opwire link 		-0 link_scale
opwire point1 		-0 show_capture
opwire cregion1		-0 cregion
opwire cregion   	-1 show_capture

opset -d on -C on -p on link_and_capture

if ($arg3 == 1) then
    opset -r on link_scale
else
    opset -r on point1
endif

opcf ..
