#ifndef __math_h__
#define __math_h__

/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	math.h (VEX)
 *
 * COMMENTS:	This include file contains useful math constants
 */

#define M_E		2.7182818
#define M_LN10		2.3025850
#define M_LN2		0.6931471
#define M_LOG10E	0.4342944
#define M_LOG2E		1.4426950
#define M_PI		3.1415926
#define M_PI_2		1.5707963
#define M_PI_4		0.7853981
#define M_SQRT1_2	0.7071067
#define M_SQRT2		1.4142135
#define M_TOLERANCE	0.0001

#if !environment(VEX_STRICT_COMPILE) && !defined(VEX_STRICT_COMPILE)
    // If not strict compiling, then allow for simpler naming conventions for
    // the math constants.
    // However, we do not allow for the simpler form of E since this conflicts
    // with global variables in some contexts.

    #define LN10	M_LN10
    #define LN2		M_LN2
    #define LOG10E	M_LOG10E
    #define LOG2E	M_LOG2E
    #define PI		M_PI
    #define PI_2	M_PI_2
    #define PI_4	M_PI_4
    #define SQRT1_2	M_SQRT1_2
    #define SQRT2	M_SQRT2
    #define TOLERANCE	M_TOLERANCE
#endif

// Defines for the maketransform() VEX function.  The function takes two
// integer arguments which determine the order that the transform gets made.
#define XFORM_SRT	0	// Scale, Rotate, Translate
#define XFORM_STR	1	// Scale, Translate, Rotate
#define XFORM_RST	2	// Rotate, Scale, Translate
#define XFORM_RTS	3	// Rotate, Translate, Scale
#define XFORM_TSR	4	// Translate, Scale, Rotate
#define XFORM_TRS	5	// Translate, Rotate, Scale

#define XFORM_XYZ	0	// Rotate order X, Y, Z
#define XFORM_XZY	1	// Rotate order X, Z, Y
#define XFORM_YXZ	2	// Rotate order Y, X, Z
#define XFORM_YZX	3	// Rotate order Y, Z, X
#define	XFORM_ZXY	4	// Rotate order Z, X, Y
#define XFORM_ZYX	5	// Rotate order Z, Y, X

#endif
