/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Mark Alexander
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada	 M5V 3E7
 *	416-504-9876
 *
 * NAME:	CHOP_FifthGlove.h ( Channel Operator Library, C++)
 *
 * COMMENTS:
 *
 *	Interface CHOP for the 5D Data glove
 */
#ifndef __CHOP_FifthGlove__
#define __CHOP_FifthGlove__

#define NUM_GLOVE_CHANNELS	7

#define ARG_5GLOVE_PORT		(myParmBase + 0)
#define ARG_5GLOVE_CUSTOM_PORT	(myParmBase + 1)
#define ARG_5GLOVE_ORIENTATION	(myParmBase + 2)
#define ARG_5GLOVE_PITCH_SCALE	(myParmBase + 3)
#define ARG_5GLOVE_ROLL_SCALE	(myParmBase + 4)
#define ARG_5GLOVE_FINGER_SCALE	(myParmBase + 5)
#define ARG_5GLOVE_ACTIVE	(myParmBase + 6)
#define ARG_5GLOVE_CALIBRATE	(myParmBase + 7)

#define ARG_5GLOVE_PITCH_NAME	(myParmBase + 8)
#define ARG_5GLOVE_ROLL_NAME	(myParmBase + 9)
#define ARG_5GLOVE_THUMB_NAME	(myParmBase +10)
#define ARG_5GLOVE_FINGER1_NAME	(myParmBase +11)
#define ARG_5GLOVE_FINGER2_NAME	(myParmBase +12)
#define ARG_5GLOVE_FINGER3_NAME	(myParmBase +13)
#define ARG_5GLOVE_FINGER4_NAME	(myParmBase +14)
#define ARG_5GLOVE_SAMPLE_RATE	(myParmBase +15)

class UT_Serial;

class CHOP_FifthGlove : public CHOP_Node
{
public:


    static OP_Node		*myConstructor(OP_Network*, const char *,
							    OP_Operator *);
    static OP_TemplatePair	 myTemplatePair;
    static OP_VariablePair	 myVariablePair;
    static PRM_Template		 myTemplateList[];
    static CH_LocalVariable	 myVariableList[];

    OP_ERROR			 cookMyChop(OP_Context &context);
    virtual int                  usesScope() const { return 0; }
    virtual unsigned		 disableParms();
protected:

				 CHOP_FifthGlove(OP_Network	 *net, 
					     const char *name,
					     OP_Operator *op);
    virtual			~CHOP_FifthGlove();

    virtual OP_ERROR		 save(ostream &os, const OP_SaveFlags &flags,
				      const char *pathPrefix);

    virtual OP_ERROR		 load(istream &is, int binary,
				      const char *extension,int fileSize,
				      const char *path);
private:

    void	setupClip(OP_Context &context);
    int		openFifthGlovePort();
    int		getRecord(char *buffer, int size, char header);

    int		getSerialName();
    void	getFifthGloveData(OP_Context &context);
    int		getFifthGloveRecord(float &pitch,float &roll, float &f0, float &f1,
			       float &f2, float &f3, float &f4, float t);

    int		calibrationScale(int index, int value);
    void	getCalibration(int p, int r, int f0, int f1, int f2, int f3,
			       int f4);

    // page 1
    int		PORT()
		 { return evalInt(ARG_5GLOVE_PORT,0,0); }

    void	CUSTOM_PORT(UT_String &str)		
		 { evalString(str, ARG_5GLOVE_CUSTOM_PORT, 0, 0); }

    int		ORIENTATION()
		 { return evalInt(ARG_5GLOVE_ORIENTATION,0,0); }

    float	PITCH_SCALE(float t)
		 { return evalFloat(ARG_5GLOVE_PITCH_SCALE,0,t); }

    float	ROLL_SCALE(float t)
		 { return evalFloat(ARG_5GLOVE_ROLL_SCALE,0,t); }

    float	FINGER_SCALE(float t)
		 { return evalFloat(ARG_5GLOVE_FINGER_SCALE,0,t); }

    int		ACTIVE()
		 { return evalInt(ARG_5GLOVE_ACTIVE,0,0); }

    int		CALIBRATE()
		{ return evalInt(ARG_5GLOVE_CALIBRATE,0,0); }
    
    // page 2
    void	PITCH_NAME(UT_String &str)
		 { evalString(str, ARG_5GLOVE_PITCH_NAME, 0, 0); }
    
    void	ROLL_NAME(UT_String &str)
		 { evalString(str, ARG_5GLOVE_ROLL_NAME, 0, 0); }

    void	THUMB_NAME(UT_String &str)
		 {  evalString(str, ARG_5GLOVE_THUMB_NAME, 0, 0); }

    void	FINGER1_NAME(UT_String &str)
		 {  evalString(str, ARG_5GLOVE_FINGER1_NAME, 0, 0); }

    void	FINGER2_NAME(UT_String &str)
		 {  evalString(str, ARG_5GLOVE_FINGER2_NAME, 0, 0); }

    void	FINGER3_NAME(UT_String &str)
		 {  evalString(str, ARG_5GLOVE_FINGER3_NAME, 0, 0); }

    void	FINGER4_NAME(UT_String &str)
		 {  evalString(str, ARG_5GLOVE_FINGER4_NAME, 0, 0); }

    float	SAMPLE_RATE(float t)
		 { return evalFloat(ARG_5GLOVE_SAMPLE_RATE,0,t); }

    // glove tracks
    CL_Track	*myPitch;
    CL_Track	*myRoll;
    CL_Track	*myThumb;
    CL_Track	*myFinger1;
    CL_Track	*myFinger2;
    CL_Track	*myFinger3;
    CL_Track	*myFinger4;
    int		 myIsCalibrating;
    float	 myMinCalibration[NUM_GLOVE_CHANNELS];
    float	 myMaxCalibration[NUM_GLOVE_CHANNELS];
    UT_String	 mySerialName; 

    UT_Serial	*mySerial;
    int		mySerialPort;
    int		myActiveState;
};

#endif
