/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	OBJ_Shake.h (Custom Library, C++)
 *
 * COMMENTS:    This custom object defines a geometry object that 
 *		has parameters which define a frame-dependent jitter
 *		in its translation. This can be used to make earth-quake
 *		type effects. No doubt next year will be the year of the
 *		Earthquake movie.
 */

#ifndef __OBJ_Shake__
#define __OBJ_Shake__

#include <OBJ/OBJ_Geometry.h>

class OBJ_Shake : public OBJ_Geometry
{
public:
				 OBJ_Shake(OP_Network *net,
					   const char *name,
					   OP_Operator *op);
    virtual			~OBJ_Shake();

    static OP_Node              *myConstructor(OP_Network *net,
					       const char *name,
					       OP_Operator *entry);

    static OP_TemplatePair	*buildTemplatePair(OP_TemplatePair *prevstuff);

    float  JX(float t)	{ return evalFloat("jitter", &shakeIndirect[0], 0, t); }
    float  JY(float t)	{ return evalFloat("jitter", &shakeIndirect[1], 1, t); }
    float  JZ(float t)	{ return evalFloat("jitter", &shakeIndirect[2], 2, t); }

protected:
    virtual int			 applyInputIndependentTransform(
				    OP_Context &context, UT_DMatrix4 &mat);
    virtual OP_ERROR		 cookMe(OP_Context &context);

private:
    static int			*shakeIndirect;
};

#endif
