/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 */

#ifndef __POP_RadialBirth_h__
#define __POP_RadialBirth_h__

#include <POP/POP_Node.h>

enum POP_RadialBirthIndex
{
    IRB_ACTIVATE,
    IRB_CENTER,
    IRB_DISTANCE,
    IRB_SPEED,
    IRB_BIRTHRATE,
    IRB_GROUP,
    IRB_LIFE,
    IRB_LIFEVAR,
    IRB_ORIGININDEX
};

class POP_RadialBirth : public POP_Node
{
public:

    static OP_Node*	myConstructor 	(OP_Network* net, const char* name, 
				         OP_Operator* entry);

    static PRM_Template	myTemplateList[];

protected:
	     POP_RadialBirth (OP_Network* net, const char* name, 
			      OP_Operator* entry);
    virtual ~POP_RadialBirth (void);

    virtual unsigned	disableParms	(void);
    virtual OP_ERROR 	cookPop		(OP_Context &context);
    virtual void        addAttrib       (void* userdata);

private:

    float  ACTIVATE    (float t) { FLOAT_PARM("activate", IRB_ACTIVATE, 0, t) };
    float  CENTERX     (float t) { FLOAT_PARM("center", IRB_CENTER, 0, t) };
    float  CENTERY     (float t) { FLOAT_PARM("center", IRB_CENTER, 1, t) };
    float  CENTERZ     (float t) { FLOAT_PARM("center", IRB_CENTER, 2, t) };
    float  DISTANCEMIN (float t) { FLOAT_PARM("distance", IRB_DISTANCE, 0, t) };
    float  DISTANCEMAX (float t) { FLOAT_PARM("distance", IRB_DISTANCE, 1, t) };
    float  SPEEDMIN    (float t) { FLOAT_PARM("speed", IRB_SPEED, 0, t) };
    float  SPEEDMAX    (float t) { FLOAT_PARM("speed", IRB_SPEED, 1, t) };
    int    BIRTHRATE   (float t) { INT_PARM("birthrate", IRB_BIRTHRATE, 0, t) };
    void   GROUP       (UT_String &s) { STR_PARM("group", IRB_GROUP, 0, 0) };
    float  LIFE        (float t) { FLOAT_PARM("life", IRB_LIFE, 0, t) };
    float  LIFEVAR     (float t) { FLOAT_PARM("lifevar", IRB_LIFEVAR, 0, t) };
    int    ORIGININDEX (void)    
			{ INT_PARM("originindex", IRB_ORIGININDEX, 0, 0) };

    int*	myIndirect;

    float	getLifetime    (POP_ContextData* data);
    int  	getOriginIndex (POP_ContextData* data);

    void        setAttrib (POP_ContextData* data, GEO_Point* ppt,
                           UT_Vector3* vel, void* userdata);

    float	myLife;
    float	myLifeVar;
    int  	myOriginIndex;
    UT_Vector3	myCenter;
    float	mySpeedMin;
    float	mySpeedMax;
};

#endif
