/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 */

#ifndef __POP_SpotLight_h__
#define __POP_SpotLight_h__

// If the POP is allowed to access the local variables defined for 
// particles, need to use the POP_LocalVar class.
#include <POP/POP_LocalVar.h>

enum POP_SpotLightIndex
{
    ISL_ACTIVATE,
    ISL_SOURCE,
    ISL_CENTER
};

class POP_SpotLight : public POP_LocalVar
{
public:

    static OP_Node*	myConstructor 	(OP_Network* net, const char* name, 
				         OP_Operator* entry);

    static PRM_Template		myTemplateList[];
    static OP_TemplatePair	myTemplatePair;
    static OP_VariablePair	myVariablePair;

protected:
	     POP_SpotLight (OP_Network* net, const char* name, 
			    OP_Operator* entry);
    virtual ~POP_SpotLight (void);

    virtual OP_ERROR 	cookPop		(OP_Context &context);
    virtual void        addAttrib       (void* userdata);

private:

    float  ACTIVATE (float t) { FLOAT_PARM("activate", ISL_ACTIVATE, 0, t) };
    void   SOURCE  (UT_String &s) { STR_PARM("source", ISL_SOURCE, 0, 0) };
    float  CENTERX (float t) { FLOAT_PARM("center", ISL_CENTER, 0, t) };
    float  CENTERY (float t) { FLOAT_PARM("center", ISL_CENTER, 1, t) };
    float  CENTERZ (float t) { FLOAT_PARM("center", ISL_CENTER, 2, t) };

    // Variables that are evaluated per particle need to have these
    // definitions.
    POP_FPARM(myCenterX, getCenterX);
    POP_FPARM(myCenterY, getCenterY);
    POP_FPARM(myCenterZ, getCenterZ);

    int*	myIndirect;

    void        changePoint (GEO_Point* ppt, POP_ContextData* data, float t,
			     POP_FParam centerx, POP_FParam centery,
			     POP_FParam centerz);
};

#undef FLOAT_PARM
#undef INT_PARM
#undef STR_PARM
#undef STR_PARM_NE

#endif
