/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 * 
 * Produced by:
 *      Side Effects Software Inc
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:        MSS_BrushHairLenSelector.C 
 *
 * COMMENTS:
 * 	Custom selector for the BrushHairLen SOP.
 */

#include <UT/UT_DSOVersion.h>
#include <SOP/SOP_BrushBase.h>
#include <BM/BM_ResourceManager.h>
#include "MSS_BrushHairLenSelector.h"

void
newSelector(BM_ResourceManager * /*m*/)
{
    PI_SelectorTemplate		*sel;
    OP_Operator			*op;
    
    // We create a PI_SelectorTemplate here to represent our new selector
    // As we want to use this with our SOP, and as the selector .dso
    // loads after the sop .dso, we also bind the selector here.

    // Create & register the selector.
    sel = new PI_SelectorTemplate("brushhairlenselect", // Internal name
				  "Brush Hair Selector", // English name
				  "Sop");		// Where it works.
    sel->constructor((void *) &MSS_BrushHairLenSelector::ourConstructor);
    sel->data(OP3DthePrimSelTypes);
    PIgetResourceManager()->registerSelector(sel);

    // Bind the selector.
    op = OP_Network::getOperatorTable(SOP_TABLE_NAME)->getOperator("proto_brushhairlen");
    if (!op)
    {
	UT_ASSERT(!"Could not find required op!");
	return;
    }

    PIgetResourceManager()->bindSelector(
	    op,				// OP_Operator to bind to.
	    "brushhairlenselect",	// Internal name of selector.
	    "Brush Hair Selector",	// English name of selector.
	    "Select the area to grow hair from.",	// Prompt.
	    "group",			// Parameter to write group to.
	    0,				// Input number to wire up.
	    1,				// 1 means this input is required.
	    "0x000000ff",		// Prim/point mask selection.
	    0,				// Can the user drag the object?
	    0,				// Optional menu.
	    0,				// Must * be used to select all?
	    false);			// true if this from a ScriptOp.
}

MSS_BrushHairLenSelector::MSS_BrushHairLenSelector(OP3D_View &viewer, 
				     PI_SelectorTemplate &templ)
    : MSS_ReusableSelector(viewer, templ, "proto_brushhairlen", "group", 
			    0, 		// No group type field.
			    true)	// We want to allow blank group names.
{
    // Nothing needed.
}

MSS_BrushHairLenSelector::~MSS_BrushHairLenSelector()
{
    // Nothing needed.
}

BM_InputSelector *
MSS_BrushHairLenSelector::ourConstructor(BM_View &viewer,
				 PI_SelectorTemplate &templ)
{
    return new MSS_BrushHairLenSelector((OP3D_View &)viewer, templ);
}

const char *
MSS_BrushHairLenSelector::className() const
{
    return "MSS_BrushHairLenSelector";
}
