/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects Software Inc.
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:	SOP Library (C++)
 *
 * COMMENTS:
 */

#ifndef __SOP_CopRaster_h__
#define __SOP_CopRaster_h__

#include <UT/UT_String.h>
#include <IMG/IMG_Raster.h>
#include <SOP/SOP_Node.h>

#define FLT_PARM(name, idx, vi, t)	\
		return evalFloat(name, &myIndirect[idx], vi, t);
#define INT_PARM(name, idx, vi, t)	\
		return evalInt(name, &myIndirect[idx], vi, t);
#define STR_PARM(name, idx, vi, t)	\
		evalString(str, name, &myIndirect[idx], vi, (float)t);

class SOP_CopRaster : public SOP_Node
{
public:
    virtual unsigned	 disableParms();
    virtual void	 resolveObsoleteParms();

    static OP_Node	*myConstructor(OP_Network*, const char *, OP_Operator*);
    static PRM_Template	 myTemplateList[];
    static PRM_Template	 myObsoleteList[];

    // This static methods is used to build a menu for the UI.
    static void	buildColorMenu(void *data, PRM_Name *, int,
				const PRM_SpareData *, PRM_Parm *);

protected:
	     SOP_CopRaster(OP_Network*, const char *, OP_Operator*);
    virtual ~SOP_CopRaster();

    virtual OP_ERROR		 cookMySop(OP_Context &context);

    // Splits a full cop2 path into the net and node portions.
    void	splitCOP2Path(const char *path, UT_String &net,
			      UT_String &node);

    // This takes a path relative to this node, and finds the full path
    // that should be sent to the CopResolver.  Flag dependent is set if
    // an interest in the flag of the destination should be set.
    int		getFullCOP2Path(const char *relpath, UT_String &fullpath,
				int &flagdependent);

private:
    int			updateRaster(float t);

    UT_String		myCurrentName;
    IMG_Raster		myRaster;

    int		USEDISK()			{ INT_PARM("usedisk", 1,0,0) }
    void	COPPATH(UT_String &str, float t){ STR_PARM("coppath", 2,0,t) }
    void	CPLANE(UT_String &str, float t) { STR_PARM("copcolor",3,0,t) }
    void	FNAME(UT_String &str, float t)	{ STR_PARM("file",    5,0,t) }
    float	COPFRAME(float t)		{ FLT_PARM("copframe",6,0,t) }

    static int	*myIndirect;

    // This variable is used together with the call to the "checkInputChanged"
    // routine to notify the handles (if any) if the input has changed.
    GU_DetailGroupPair	 myDetailGroupPair;
};

#undef FLT_PARM
#undef INT_PARM
#undef STR_PARM

#endif
