/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:	SOP library (C++)
 *
 * COMMENTS:	Flatten SOP
 */


#ifndef __SOP_Flatten_h__
#define __SOP_Flatten_h__

#include <SOP/SOP_Node.h>

class SOP_Flatten : public SOP_Node
{
public:
	     SOP_Flatten(OP_Network *net, const char *name, OP_Operator *op);
    virtual ~SOP_Flatten();

    // This method is created so that it can be called by handles.  It only
    // cooks the input group of this SOP.  The geometry in this group is
    // the only geometry manipulated by this SOP.
    virtual OP_ERROR		 cookInputGroups(OP_Context &context, 
						int alone = 0);

    static PRM_Template		 myTemplateList[];
    static OP_Node		*myConstructor(OP_Network*, const char *,
							    OP_Operator *);

protected:
    virtual unsigned		 disableParms();
    virtual const char          *inputLabel(unsigned idx) const;

    // Method to cook geometry for the SOP
    virtual OP_ERROR		 cookMySop(OP_Context &context);

    // This method is used to generate geometry for a "guide".  It does
    //	not have to be defined.
    virtual OP_ERROR             cookMyGuide1(OP_Context &context);

private:
    void	getGroups(UT_String &str)	{ evalString(str, 0, 0, 0); }
    float	DIST(float t)			{ return evalFloat(1, 0, t); }
    int		DIRPOP(void)			{ return evalInt(2, 0, 0); }
    int		ORIENT()			{ return evalInt(3, 0, 0); }
    float	NX(float t)			{ return evalFloat(4, 0, t); }
    float	NY(float t)			{ return evalFloat(4, 1, t); }
    float	NZ(float t)			{ return evalFloat(4, 2, t); }

    // This variable is used together with the call to the "checkInputChanged"
    // routine to notify the handles (if any) if the input has changed.
    GU_DetailGroupPair	 myDetailGroupPair;

    // This is the group of geometry to be manipulated by this SOP and cooked
    // by the method "cookInputGroups".
    const GB_PointGroup	*myGroup;
};

#endif
