/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Side Effects Software Inc.
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	SOP Library (C++)
 *
 * COMMENTS: 
 *	This is an example of an implementation of the Object Merge
 * SOP to demonstrate using DYNAMIC_PATHs, MULTIPARMS, and proper
 * cooking of other SOPs.
 *
 */

#ifndef __SOP_HDKObject_h__
#define __SOP_HDKObject_h__

#include <SOP/SOP_Node.h>

class SOP_HDKObject : public SOP_Node
{
public:
	    SOP_HDKObject(OP_Network *net, const char *name, 
			  OP_Operator *entry);
    virtual ~SOP_HDKObject();

    virtual unsigned	 disableParms();
    static OP_Node		*myConstructor(OP_Network *net, const char *name, OP_Operator *entry);
    static PRM_Template		 myTemplateList[];
    static PRM_Template		 myObsoleteList[];

    // Because the object merge can reference an op chain which has a 
    // subnet with different D & R we must follow all our node's
    // d & r status
    virtual int			 getDandROpsEqual();
    virtual int			 updateDandROpsEqual(int = 1) 
				 { return getDandROpsEqual(); }

    int 	NUMOBJ() { return evalInt("numobj", 0, 0.0f); }
    void	setNUMOBJ(int num_obj)
		    { setInt("numobj", 0, 0.0f, num_obj); }

    int		ENABLEMERGE(int i)
		{
		    return evalIntInst("enable%d", &i, 0, 0.0f);
		}
    void	setENABLEMERGE(int i, int val)
		{
		    setIntInst(val, "enable%d", &i, 0, 0.0f);
		}

    void	SOPPATH(UT_String &str, int i, float t)
		{
		    evalStringInst("objpath%d", &i, str, 0, t);
		}
    void	setSOPPATH( UT_String &str, int i, float t)
		{
		    setStringInst(str, "objpath%d", &i, 0, t);
		}

    void	XFORMPATH(UT_String &str, float t)
		{
		    evalString(str, "xformpath", 0, t);
		}
protected:
    virtual OP_ERROR		 cookMySop(OP_Context &context);
};

#endif
