/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:	SOP library (C++)
 *
 * COMMENTS:	This is a simple particle system...  Not too much to it, but
 *		this can be used as a template for your own...
 */


#ifndef __SOP_SParticle_h__
#define __SOP_SParticle_h__

#include <SOP/SOP_Node.h>

#define INT_PARM(name, idx, vidx, t)	\
	    return evalInt(name, &myOffsets[idx], vidx, t);

#define FLT_PARM(name, idx, vidx, t)	\
	    return evalFloat(name, &myOffsets[idx], vidx, t);

class GEO_ParticleVertex;
class GEO_PrimParticle;
class GU_RayIntersect;

class SOP_SParticle : public SOP_Node
{
public:
	     SOP_SParticle(OP_Network *net, const char *name, OP_Operator *op);
    virtual ~SOP_SParticle();

    static PRM_Template		 myTemplateList[];
    static OP_Node		*myConstructor(OP_Network*, const char *,
							    OP_Operator *);

protected:
    virtual unsigned		 disableParms();
    virtual const char          *inputLabel(unsigned idx) const;

    void		birthParticle();
    int			moveParticle(GEO_ParticleVertex *vtx,
				     const UT_Vector3 &force);

    void		initSystem();
    void		timeStep(float now);

    // Method to cook geometry for the SOP
    virtual OP_ERROR		 cookMySop(OP_Context &context);

private:
    // These use defines to make it easy to add parms and remove them.
    // The evaluation routines use the indexed name lookup which is quite
    //	fast, yet easy to change indices (since the order of the indices
    //	doesn't have to be in sequential order...
    int			 RESET()	{ INT_PARM("reset", 0, 0, 0) }
    int			 BIRTH(float t)	{ INT_PARM("birth", 2, 0, t) }
    float		 FX(float t)	{ FLT_PARM("force", 1, 0, t) }
    float		 FY(float t)	{ FLT_PARM("force", 1, 1, t) }
    float		 FZ(float t)	{ FLT_PARM("force", 1, 2, t) }

    const GU_Detail	*mySource;
    int			 mySourceNum;		// Source point to birth from
    int			 mySourceVel;		// Velocity attrib in source

    GU_RayIntersect	*myCollision;

    GEO_PrimParticle	*mySystem;
    float		 myLastCookTime;	// Last cooked time
    int			 myVelocity;		// My velocity attribute
    int			 myLife;		// My life attribute

    static int		*myOffsets;
    
    // This variable is used together with the call to the "checkInputChanged"
    // routine to notify the handles (if any) if the input has changed.
    GU_DetailGroupPair	 myDetailGroupPair;
};

#endif
