/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 *
 * NAME:	standalone
 *
 * COMMENTS:	Sample stand alone application
 */

#ifndef SOLARIS
#include <getopt.h>
#endif
#include <stdlib.h>
#include <strstream.h>
#include <MOT/MOT_Director.h>

#define LINE_SIZE	1024

static void
usage(const char *program)
{
    cerr << "Usage: " << program << " [-h]\n";
    cerr << "Stand alone houdini application" << endl;
    exit(1);
}

int
main(int argc, char *argv[])
{
    int			 opt, i;
    MOT_Director	*boss;
    CMD_Manager		*cmd;
    char		 line[LINE_SIZE];

    // Do argument parsing
    while ((opt = getopt(argc, argv, "h")) != -1)
    {
	switch (opt)
	{
	    case 'h':
	    default:	usage(argv[0]);
	}
    }

    // Initialize the whole heirarchy of operators
    boss = new MOT_Director("standalone");
    OPsetDirector(boss);

    cmd = boss->getCommandManager();
    if (argc == optind)
    {
	// If we have no arguments, source 123.cmd
	cmd->sendInput("source -q 123.cmd");
    }
    else
    {
	for (i = optind; i < argc; i++)
	{
	    // Merge in using the mread command.
	    ostrstream ts(line, LINE_SIZE); 
	    ts << "mread -m * " << argv[i] << ends;
	    cmd->sendInput(line);
	}
    }

    // Now, enter the main loop
    do {
	cmd->doPrompt();
	if (!cin.getline(line, LINE_SIZE))
	    break;
	cmd->sendInput(line);
    } while (cin);

    return 0;
}
